; Script to check integrated photometry against their original images
; for artifacts including cosmic rays and stars
;
; Commands:
;
;   <ENTER>: keep the current flag
;   # (0-5): set flag to current image
;   b      : back to previous image
;   a      ; compare with previous sequence
;   s      ; compare with next sequence
;   w      ; save flag to photometry table without exiting program
;   e      : end the processing and save change
;   q      ; quick the program without saving change
;
; History:
;
; 9/9/2010, created by JYL @UMd
;   Derived from plot_photo.pro
; 9/22/2010, modified by JYL @UMd
;   * Include Farnham's mri_seq_compare.pro function.
;   * Add commands a and s.
; 9/23/2010, modified by JYL @UMd
;   * Chang the scale of reference lightcurve plot to show the
;     scatters of photometry from one exposure ID better.
; 9/24/2010, modified by JYL @UMd
;   * Add command w to save flag without exiting
;   * Add tolerance to unexpected command
; 10/13/2010, modified by JYL @UMd
;   * Modified to accomodate 10-aperture photometry from 1-pixel to
;     19-pixel apertures
;11/15/2010, modified by Jade @UMD
;   flag of 5 added to indicated smeared appearance in frame (& 4, means over-saturated)


; photometry table: table of preliminary photometry values measured in radii 1-10 pixels
photfile = '~/IDLWorkspace80/JYdailyLCsSDC/photometry/aperdotpro_phot.tab'
; imagelist file with centroid information (from Farnham): widest_ap_first.tab 
geofile= '~/Desktop/products3/widest_ap_full.tab'
; data directory: location of image fits files that have been cleaned of cosmic rays 
datadir = '~/IDLWorkspace80/JYdailyLCsSDC/data_final/crrej/original_crrej/'

;--------------------------------------------


; read in photometry table
readcol, photfile,files,jdmid,flag,filter,hed,spc,ftot1,ftot2,ftot3,ftot4,ftot5,ftot6,ftot7,ftot8,ftot9,ftot10,format='a,d,i,a,d,f,f,f,f,f,f,f,f,f,f,f',skipline=3,/silent

;arrays holding photometry
ftot = [[ftot1],[ftot2],[ftot3],[ftot4],[ftot5],[ftot6],[ftot7],[ftot8],[ftot9],[ftot10]]
ferr = [[ferr1],[ferr2],[ferr3],[ferr4],[ferr5],[ferr6],[ferr7],[ferr8],[ferr9],[ferr10]]
naper = 10
;aperture pixel radii
aper = [1.5, 2.5, 3.5, 4.5, 5.5, 6.5, 7.5, 8.5, 9.5]

; read in geometric data
readcol,geofile,file,jd,doy,filt,xc,yc,format='a,d,i,a,f,f',/silent,skipline=2
stop
ww = where(xc ne 0 and yc ne 0)
flag1 = flag

nimgs = n_elements(ww)

; set up display windows
window,31,xs=500,ys=800,xpos=0,ypos=0   ; window 31 to plot photometry
window,30,xs=512,ys=512,xpos=501,ypos=0   ; display image in original size
window,29,xs=256,ys=256,xpos=1013,ypos=0   ; display 8x zoomed image

; change to log scale
ftot1 = -alog10(ftot)*2.5
; x-axis
dt = jdmid+0.5-2455197

backed = 0   ; flag to backup existing photometry table

; start checking images
in = ''
expid0 = ''

for i=nimgs-1,0,-1 do begin
;for i=(where(files[ww] eq 'mv0343556854_4300017_001_rr.fit'))[0],0,-1 do begin

;  i = where(files[ww] eq 'mv0337878414_4000026_001_rr.fit')
;  i = i[0]

  ; plot photometry as references
  expid = datadir+string(doy[i],format='(i3.3)')+'/'+files[ww[i]]
  if expid ne expid0 then begin
    xran = [(dt[ww[i]]-2/24.)>min(dt),(dt[ww[i]]+2/24.)<max(dt)]
    if xran[0] eq min(dt) then xran[1]=xran[0]+4/24.
    if xran[1] eq max(dt) then xran[0]=xran[1]-4/24.
    yran1 = avg(ftot1[ww[i-1:i],0])+[0.5,-0.5]
    yran2 = avg(ftot1[ww[i-1:i],1])+[0.5,-0.5]
    yran3 = avg(ftot1[ww[i-1:i],2])+[0.5,-0.5]
    yran4 = avg(ftot1[ww[i-1:i],3])+[0.5,-0.5]
    yran5 = avg(ftot1[ww[i-1:i],4])+[0.5,-0.5]
    expid0 = expid
  endif

  wset,31
  !p.multi=[0,1,5]
  plot,dt[ww],ftot1[ww,0],ps=4,xran=xran,charsize=1.5,yran=yran1
  oplot,[dt[ww[i]]],[ftot1[ww[i],0]],ps=7,symsize=2
  plot,dt[ww],ftot1[ww,1],ps=4,xran=xran,charsize=1.5,yran=yran2
  oplot,[dt[ww[i]]],[ftot1[ww[i],1]],ps=7,symsize=2
  plot,dt[ww],ftot1[ww,2],ps=4,xran=xran,charsize=1.5,yran=yran3
  oplot,[dt[ww[i]]],[ftot1[ww[i],2]],ps=7,symsize=2
  plot,dt[ww],ftot1[ww,3],ps=4,xran=xran,charsize=1.5,yran=yran4
  oplot,[dt[ww[i]]],[ftot1[ww[i],3]],ps=7,symsize=2
  plot,dt[ww],ftot1[ww,4],ps=4,xran=xran,charsize=1.5,yran=yran5
  oplot,[dt[ww[i]]],[ftot1[ww[i],4]],ps=7,symsize=2
  !p.multi=0

  ; read in image
  ;img = readfits(datadir+files[ww[i]],hdr,/silent)
  img = readfits(datadir+string(doy[i],format='(i3.3)')+'/'+files[ww[i]],hdr,/silent)
print,xc[ww[i]],yc[ww[i]],size(img,/dimensions)
  ; display full frame image
  wset,30
;  maxdisp = max(img[xc[ww[i]]-6:xc[ww[i]]+6,yc[ww[i]]-6:yc[ww[i]]+6])
  maxdisp = max(img[xc[ww[i]]:xc[ww[i]],yc[ww[i]]:yc[ww[i]]])
  tvscl, img<maxdisp>0

  ; display zoomed in images
  wset,29
  xc0 = floor(xc[ww[i]])
  xc1 = xc[ww[i]]-xc0+0.5
  yc0 = floor(yc[ww[i]])
  yc1 = yc[ww[i]]-yc0+0.5
  disp = rebin(img[xc0-16:xc0+15,yc0-16:yc0+15],256,256,/sam)
  makenxy,0,255,256,0,255,256,xarr,yarr
  xarr -= (128+xc1*8)
  yarr -= (128+yc1*8)
  rarr = sqrt(xarr*xarr+yarr*yarr)
  disp[where(rarr gt 3*8-1 and rarr lt 3*8)] = maxdisp
  disp[where(rarr gt 5*8-1 and rarr lt 5*8)] = maxdisp
;  disp[where(rarr gt 10*8-1 and rarr lt 10*8)] = maxdisp
  tvscl,disp<maxdisp>0

  read,files[ww[i]]+', current flag '+string(flag1[ww[i]],format='(i1)')+' : (#/b/a/s/w/e/q/<enter>): ',in
  case in of
    'b' : i+=2   ; go to previous image
    'e' : begin    ; quit and save change
            print
            print, 'Changes are saved.'
            goto, quit
          end
    'q' : begin   ; quit without saving
            print
            print, 'Changes are lost.'
            goto, the_end
          end
    's' : begin    ; compare with next sequence
            mri_seq_compare, files[ww[i]],imdir=datadir,cendir=cendir,exp=4,uhist=99
            i++
          end
    'a' : begin    ; compare with previous sequence
            mri_seq_compare, files[ww[i]],imdir=datadir,cendir=cendir,/prev,exp=4,uhist=99
            i++
          end
    'w' : begin    ; save results without exiting
            ; backup existing photometry table if not yet
            if not backed then begin
              backfile = strmid(photfile,0,strpos(photfile,'.tab')+1)+'bak'
              if file_search(backfile) ne '' then file_delete,backfile
              file_move, photfile,backfile
              backed = 1
            endif
            flag = flag1
            openw,f,photfile,/get_lun
            printf,f,'          File                             JD_mid      Filter    T_exp Flag                               Flux (W/[m^2 sr um])                                          Error (W/[m^2 sr um])'
            tblhdr = '            1-pix'
            for j=0,naper-2 do tblhdr+='        '+string(aper[j]*2,format='(i2)')+'-pixel'
            printf,f,'                                                                  (s)'+tblhdr+tblhdr
            line = '-------------------------------------------------------'
            for j=0,(naper-1)*2 do line+='------------------'
            printf,f,line
            fmtstr = '(a35,f18.7,a8,f9.2,i4,'+strtrim(string((naper+1)*2,format='(i2)'),2)+'g16.8)'
            for j=0,n_elements(files)-1 do printf,f,files[j],jdmid[j],filter[j],texp[j],flag[j],ftot[j,*],ferr[j,*],format=fmtstr
            close,f & free_lun,f
            print, 'Flags saved.'
            i++
          end
    else: if in ne '' then begin
            if (where(in eq ['0','1','2','3','4','5']))[0] ne -1 then begin
              reads,in,f
              flag1[ww[i]] = f
            endif else i++
          endif
  endcase

endfor

stop
quit:

; backup existing photometry table if not yet
if not backed then begin
  backfile = strmid(photfile,0,strpos(photfile,'.tab')+1)+'bak'
  if file_search(backfile) ne '' then file_delete,backfile
  file_move, photfile,backfile
  backed = 1
endif

; save results
flag = flag1
openw,f,photfile,/get_lun
printf,f,'          File                             JD_mid      Filter    T_exp Flag                               Flux (W/[m^2 sr um])                                          Error (W/[m^2 sr um])'
tblhdr = '            1-pix'
for i=0,naper-2 do tblhdr+='        '+string(aper[i]*2,format='(i2)')+'-pixel'
printf,f,'                                                                  (s)'+tblhdr+tblhdr
line = '-------------------------------------------------------'
for i=0,(naper-1)*2 do line+='------------------'
printf,f,line
fmtstr = '(a35,d18.7,a8,f9.2,i4,'+strtrim(string((naper+1)*2,format='(i2)'),2)+'g16.8)'
for i=0,n_elements(files)-1 do printf,f,files[i],jdmid[i],filter[i],texp[i],flag[i],ftot[i,*],ferr[i,*],xc[i],format=fmtstr
close,f & free_lun,f

the_end:

wdelete, 31
wdelete, 30
wdelete, 29

end
