; Created 5/22/2012 to conduct photometry on cosmic ray rejected & gap-corrected images
;background subtraction & scaling excluded
;both photometry methods are included I. aperdotpro method & II. radial profiling method
;only measurement in specific aperture pixel radii
;independent photometric  uncertainties are also calculated 
;we perform photometry on uncorrected image in order to compute
;the independent photometric  uncertainty associated with making this
;correction (gap correction)
;---------------------------------------------------------------------
;top level directory
app='/{partiaion}/jwilliams'
;original cosmic ray rejected image directory
dir= app+'/IDLWorkspace80/JYdailyLCsSDC/data_final/crrej/original_crrej/'
;gap-corrected cosmic ray rejected image directory 
newdir= app+'/IDLWorkspace80/JYdailyLCsSDC/data_final/crrej/compress_gap/'
;image list file with adjusted centroids 
inputfile= app+'/products_PDS/widest_ap.tab' 
;official image list file with original measured centroids
inputfile_orig= app+'/products_PDS/widest_ap_first.tab' 
;----------------------------------------------------------------------
readcol,inputfile,file,jd,doy,filter,xc,yc,frame,flag,hed,spc,pa,ap,format='a,d,i,a,f,f,i,i,d,d,f,i',skip=2,/silent
readcol,inputfile_orig,file_o,jd_o,doy_o,filter_o,xc_o,yc_o,frame_o,flag_o,hed_o,spc_o,pa_o,ap_o,format='a,d,i,a,f,f,i,i,d,d,f,i',skip=2,/silent
;----------------------------------------------------------------------
;constants
ln=n_elements(file) & gain=28.5 ;# of images, instrument gain
;uncertainties- err1: gap correction, err2:Poisson

;aperdotpro pixel photometry arrays
orig_adp_pix= fltarr(ln,30) & phot_adp_pix= fltarr(ln,30) & err1_adp_pix= fltarr(ln,30) & err2_adp_pix= fltarr(ln,30) & e_adp_pix= fltarr(ln,30)

;arrays for horizontal/vertical distance b/t centroid & CCD center line
h_dist= fltarr(ln) & v_dist= fltarr(ln)
;----------------------------------------------------------------------
;----------------------------------------------------------------------
;loop through images
;----------------------------------------------------------------------
;----------------------------------------------------------------------
for i=0,ln-1 do begin
   print,strtrim(string(ln-i),2)+'...'
   ;read original uncorrected image
   img=readfits(dir+strtrim(string(doy[i]),2)+'/'+file[i],hdr,/silent)
   ;read gap-corrected image
   new=readfits(newdir+strtrim(string(doy[i]),2)+'/'+file[i],hdr,/silent)
   radcalv = sxpar(hdr,'RADCALV')   ; rad cal constant
   texp = sxpar(hdr,'INTTIME')/1000. ; exposure time in sec
;convert to DN for Poisson error
   img /= (radcalv/texp) & new /= (radcalv/texp) ;image remains converted to DN
;------------------------------------------------
;centroid distance to CCD center lines
;------------------------------------------------
   line= frame[i]/2-.5 ;locations of CCD horizontal and vertical center lines
   ;determine vertical distance to center line, v_dist
    v_dist[i]= yc_o[i]-line
   ;determine horizontal distance to center line, h_dist
    h_dist[i]= xc_o[i]-line
;------------------------------------------------
;Computing Photometry
;------------------------------------------------
;I. aperdotpro pixel photometry method
   for j=0,29 do begin
      if j+1 gt ap[i] then break
;measure photometry/error of distorted image
      aper,img,xc_o[i],yc_o[i],flux_img,eflux_img,sky,skyerr,gain,j+1,[0.,0.],[0,0],/flux,setskyval=0,/silent
;measure photometry/error of corrected image
      aper,new,xc[i],yc[i],flux_new,eflux,sky,skyerr,gain,j+1,[0.,0.],[0,0],/flux,setskyval=0,/silent
;store values, converted from DN to flux
      phot_adp_pix[i,j]= flux_new*(radcalv/texp) & orig_adp_pix[i,j]= flux_img*(radcalv/texp)  ;flux
      err1_adp_pix[i,j]= abs(flux_img-flux_new)*(radcalv/texp)  ;gap error flux
      err2_adp_pix[i,j]= eflux*(radcalv/texp)    ;Pois error flux
   endfor

;----------------------------------------------
;combine independent errors
;----------------------------------------------
e_adp_pix[i,*]=sqrt(err1_adp_pix[i,*]^2+err2_adp_pix[i,*]^2)
;----------------------------------------------
;overwrite zeros (in aperture sizes exceeding within 6 pixels of image frame)
;----------------------------------------------
z1=where(phot_adp_pix[i,*] eq 0,ct) & if ct gt 0 then phot_adp_pix[i,z1]= -1.E32
z2=where(e_adp_pix[i,*] eq 0,ct) & if ct gt 0 then e_adp_pix[i,z2]= -1.E32

;----------------------------------------------
;write data tables
;----------------------------------------------
;printing table of photometry by method I.
openw,g,app+'/Dennis_tables/adp_phot_1_33.tab',/get_lun,/append
fmtstr = '(A-29, x, F15.7, x, I3, x, A-11, 2(x, F6.2), x, I3, x, 2(x, F6.2), x, I1, x, F6.4, x, F11.2, x, F5.2, x, I3, 33(1x, E9.2E2))'
 printf,g,file[i],jd[i],doy[i],filter[i],xc_o[i],yc_o[i],frame[i],h_dist[i],v_dist[i],flag[i],hed[i],spc[i],pa[i],ap[i],phot_adp_pix[i,*]*1e-10,format=fmtstr
close,g & free_lun,g

;printing table of photometric error by method I.
openw,g,app+'/Dennis_tables/adp_err_1_33.tab',/get_lun,/append
fmtstr = '(A-29, x, F15.7, x, I3, x, A-11, 2(x, F6.2), x, I3, x, 2(x, F6.2), x, I1, x, F6.4, x, F11.2, x, F5.2, x, I3, 33(1x, E9.2E2))'
 printf,g,file[i],jd[i],doy[i],filter[i],xc_o[i],yc_o[i],frame[i],h_dist[i],v_dist[i],flag[i],hed[i],spc[i],pa[i],ap[i],e_adp_pix[i,*]*1e-10,format=fmtstr
close,g & free_lun,g

endfor
end
