; Created 5/22/2012 to conduct photometry on cosmic ray rejected & gap-corrected images
;background subtraction & scaling excluded
;both photometry methods are included I. aperdotpro method & II. radial profiling method
;only measurement in specific aperture pixel radii
;independent photometric  uncertainties are also calculated 
;we perform photometry on uncorrected image in order to compute
;the independent photometric  uncertainty associated with making this
;correction (gap correction)
;---------------------------------------------------------------------
;top level directory
app='/{partiaion}/jwilliams'
;original cosmic ray rejected image directory
dir= app+'/IDLWorkspace80/JYdailyLCsSDC/data_final/crrej/original_crrej/'
;gap-corrected cosmic ray rejected image directory 
newdir= app+'/IDLWorkspace80/JYdailyLCsSDC/data_final/crrej/compress_gap/'
;image list file with adjusted centroids 
inputfile= app+'/products_PDS/widest_ap.tab' 
;official image list file with original measured centroids
inputfile_orig= app+'/products_PDS/widest_ap_first.tab' 
;----------------------------------------------------------------------
readcol,inputfile,file,jd,doy,filter,xc,yc,frame,flag,hed,spc,pa,ap,format='a,d,i,a,f,f,i,i,d,d,f,i',skip=2,/silent
readcol,inputfile_orig,file_o,jd_o,doy_o,filter_o,xc_o,yc_o,frame_o,flag_o,hed_o,spc_o,pa_o,ap_o,format='a,d,i,a,f,f,i,i,d,d,f,i',skip=2,/silent
;----------------------------------------------------------------------
;specified aperture radii
pix=[1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 12, 14, 16, 18, 20, 25, 30, 35, 40, 45, 50, 55, 60, 80, 100, 120, 140, 160, 180, 200, 220, 240, 248]
;constants
ln=n_elements(file) & gain=28.5 ;# of images, instrument gain
;uncertainties- err1: gap correction, err2:Poisson, err3: artifact rej

;aperdotpro pixel photometry arrays
orig_adp_pix= fltarr(ln,33) & phot_adp_pix= fltarr(ln,33) & err1_adp_pix= fltarr(ln,33) & err2_adp_pix= fltarr(ln,33) & e_adp_pix= fltarr(ln,33)
;radial profiling pixel photometry arrays
orig_rp_pix= fltarr(ln,33) & phot_rp_pix= fltarr(ln,33) & err1_rp_pix= fltarr(ln,33) & err2_rp_pix= fltarr(ln,33)  & err3_rp_pix= fltarr(ln,33) & e_rp_pix= fltarr(ln,33)
;arrays for horizontal/vertical distance b/t centroid & CCD center line
h_dist= fltarr(ln) & v_dist= fltarr(ln)
;----------------------------------------------------------------------
;----------------------------------------------------------------------
;loop through images
;----------------------------------------------------------------------
;----------------------------------------------------------------------
for i=0,ln-1 do begin
;sigma values unique to image filter
   x=filter[i]
   case x of
      'CLEAR1': th=3.5
      'CN': th=3
      'C2': th=3
      'OH': th=3
      'GREEN_CONT': th=3.5
      else: th=3
   endcase
   print,strtrim(string(ln-i),2)+'...'
   ;read original uncorrected image
   img=readfits(dir+strtrim(string(doy[i]),2)+'/'+file[i],hdr,/silent)
   ;read gap-corrected image
   new=readfits(newdir+strtrim(string(doy[i]),2)+'/'+file[i],hdr,/silent)
   radcalv = sxpar(hdr,'RADCALV')   ; rad cal constant
   texp = sxpar(hdr,'INTTIME')/1000. ; exposure time in sec
;convert to DN for Poisson error
   img /= (radcalv/texp) & new /= (radcalv/texp) ;image remains converted to DN
;arrays for holding azimuthal profile, max value in non-artifact portion of profile, & mean uncertainty
   profile_pix=fltarr(250) & profile_pmax=fltarr(250) & profile_unc_pix=fltarr(250)
   profile_km=fltarr(101) & profile_kmax=fltarr(101) & profile_unc_km=fltarr(101)
;------------------------------------------------
;centroid distance to CCD center lines
;------------------------------------------------
   line= frame[i]/2-.5 ;locations of CCD horizontal and vertical center lines
   ;determine vertical distance to center line, v_dist
    v_dist[i]= yc_o[i]-line
   ;determine horizontal distance to center line, h_dist
    h_dist[i]= xc_o[i]-line
;------------------------------------------------
;Computing Photometry
;------------------------------------------------
;I. aperdotpro pixel photometry method
   for j=0,32 do begin
      if pix[j] gt ap[i] then break
;measure photometry/error of distorted image
      aper,img,xc_o[i],yc_o[i],flux_img,eflux_img,sky,skyerr,gain,pix[j],[0.,0.],[0,0],/flux,setskyval=0,/silent
;measure photometry/error of corrected image
      aper,new,xc[i],yc[i],flux_new,eflux,sky,skyerr,gain,pix[j],[0.,0.],[0,0],/flux,setskyval=0,/silent
;store values, converted from DN to flux
      phot_adp_pix[i,j]= flux_new*(radcalv/texp) & orig_adp_pix[i,j]= flux_img*(radcalv/texp)  ;flux
      err1_adp_pix[i,j]= abs(flux_img-flux_new)*(radcalv/texp)  ;gap error flux
      err2_adp_pix[i,j]= eflux*(radcalv/texp)    ;Pois error flux
   endfor
;II. radial profiling pixel photometry method
   ;measure distorted image
   sz = 1.0*size(img) 
   ttheta = 1.0*indgen(sz[1]) & ttheta = !pi*(bytscl(ttheta)/255.0) - !pi/2
   leng = n_elements(ttheta)
   hir = floor(sqrt((0.5*sz(1))^2 + (0.5*sz(2))^2))
   r = 1.0*indgen(hir)          ;array the size of radius
   cartop, img, xc_o[i],yc_o[i], max(r), r, theta, az ;conversion to polar image
   for j=0,ap[i] do begin
      col = reform(az[j,*])
      RESISTANT_Mean,col,th,mean,goodvec=kept
      knum=n_elements(kept)   ;how many kept
      if knum eq n_elements(col) then begin
         profile_pix[j]=mean & profile_pmax[j]=max(col)
         continue
      endif
      keep=col[kept] ;array of kept values   
      col[kept]=-1
      mid=median(keep,/double)
      trim=where(col ne -1.,tnum) ;indices of trimmed values
      if n_elements(col)-knum ne tnum then stop
      ;now to include values underneath median
      above=where(col[trim] gt mid) & takeback=where(col[trim] lt mid,ct)
      if ct gt 0 then begin
         keeper=[keep,col[trim[takeback]]]
         if n_elements(keeper) ne ct+knum then stop
      endif else keeper=[keep]
      profile_pix[j]=mean(keeper) & profile_pmax[j]=max(keeper)
      profile_unc_pix[j]= (profile_pmax[j]-profile_pix[j])*(double(n_elements(col)-n_elements(keeper))/double(n_elements(keeper)))
   endfor
;compute integer pixel photometry
   for k=0,32 do begin
      lim=pix[k] & ind=where(r le lim) ;gathering all aperture values down to 1pixel
      orig_rp_pix[i,k]=2.*!pi*int_tabulated(r[ind],r[ind]*profile_pix[ind]) ;integrating flux
   endfor
;measure corrected image
   sz = 1.0*size(new) 
   ttheta = 1.0*indgen(sz[1]) & ttheta = !pi*(bytscl(ttheta)/255.0) - !pi/2
   leng = n_elements(ttheta)
   hir = floor(sqrt((0.5*sz(1))^2 + (0.5*sz(2))^2))
   r = 1.0*indgen(hir)          ;array the size of radius
   cartop, new, xc[i],yc[i], max(r), r, theta, az ;conversion to polar image
   for j=0,ap[i] do begin
      col = reform(az[j,*])
      RESISTANT_Mean,col,th,mean,goodvec=kept
      knum=n_elements(kept)   ;how many kept
      if knum eq n_elements(col) then begin
         profile_pix[j]=mean & profile_pmax[j]=max(col)
         continue
      endif
      keep=col[kept] ;array of kept values   
      col[kept]=-1
      mid=median(keep,/double)
      trim=where(col ne -1.,tnum) ;indices of trimmed values
      if n_elements(col)-knum ne tnum then stop
      ;now to include values underneath median
      above=where(col[trim] gt mid) & takeback=where(col[trim] lt mid,ct)
      if ct gt 0 then begin
         keeper=[keep,col[trim[takeback]]]
         if n_elements(keeper) ne ct+knum then stop
      endif else keeper=[keep]
      profile_pix[j]=mean(keeper) & profile_pmax[j]=max(keeper)
      profile_unc_pix[j]= (profile_pmax[j]-profile_pix[j])*(double(n_elements(col)-n_elements(keeper))/double(n_elements(keeper)))
   endfor
;compute integer pixel photometry
   for k=0,32 do begin
      lim=pix[k] & ind=where(r le lim) ;gathering all aperture values down to 1pixel
      phot_rp_pix[i,k]=2.*!pi*int_tabulated(r[ind],r[ind]*profile_pix[ind]) ;integrating flux
      err3_rp_pix[i,k]=2.*!pi*int_tabulated(r[ind],r[ind]*profile_unc_pix[ind]) ;integrating art.rej. error
   endfor
;calculate error (DN's)
   err2_rp_pix[i,*]= sqrt(phot_rp_pix[i,*]/gain)
;store values, converted from DN to flux
   phot_rp_pix[i,*]*= (radcalv/texp) &  orig_rp_pix[i,*]*= (radcalv/texp) ;flux
   err1_rp_pix[i,*]= abs(orig_rp_pix[i,*]-phot_rp_pix[i,*]) ;gap error flux
   err2_rp_pix[i,*]*= (radcalv/texp) ;Pois error flux
   err3_rp_pix[i,*]*= (radcalv/texp) ;art.rej flux error

;----------------------------------------------
;combine independent errors
;----------------------------------------------
e_adp_pix[i,*]=sqrt(err1_adp_pix[i,*]^2+err2_adp_pix[i,*]^2)
e_rp_pix[i,*]=sqrt(err1_rp_pix[i,*]^2+err2_rp_pix[i,*]^2+err3_rp_pix[i,*]^2)
;----------------------------------------------
;overwrite zeros (in aperture sizes exceeding within 6 pixels of image frame)
;----------------------------------------------
z1=where(phot_adp_pix[i,*] eq 0,ct) & if ct gt 0 then phot_adp_pix[i,z1]= -1.E32
z2=where(e_adp_pix[i,*] eq 0,ct) & if ct gt 0 then e_adp_pix[i,z2]= -1.E32
z3=where(phot_rp_pix[i,*] eq 0,ct) & if ct gt 0 then phot_rp_pix[i,z3]= -1.E32
z4=where(e_rp_pix[i,*] eq 0,ct) & if ct gt 0 then e_rp_pix[i,z4]= -1.E32
endfor

;----------------------------------------------
;write data tables
;----------------------------------------------
;printing table of photometry by method I.
openw,g,app+'/Nov1_delivery/codes/photometry/tables/adp_phot.tab',/get_lun
fmtstr = '(A-29, x, F15.7, x, I3, x, A-11, 2(x, F6.2), x, I3, x, 2(x, F6.2), x, I1, x, F6.4, x, F11.2, x, F5.2, x, I3, 33(1x, E9.2E2))'
for i=0,ln-1 do printf,g,file[i],jd[i],doy[i],filter[i],xc_o[i],yc_o[i],frame[i],h_dist[i],v_dist[i],flag[i],hed[i],spc[i],pa[i],ap[i],phot_adp_pix[i,*]*1e-10,format=fmtstr
close,g & free_lun,g

;printing table of photometric error by method I.
openw,g,app+'/Nov1_delivery/codes/photometry/tables/adp_err.tab',/get_lun
fmtstr = '(A-29, x, F15.7, x, I3, x, A-11, 2(x, F6.2), x, I3, x, 2(x, F6.2), x, I1, x, F6.4, x, F11.2, x, F5.2, x, I3, 33(1x, E9.2E2))'
for i=0,ln-1 do printf,g,file[i],jd[i],doy[i],filter[i],xc_o[i],yc_o[i],frame[i],h_dist[i],v_dist[i],flag[i],hed[i],spc[i],pa[i],ap[i],e_adp_pix[i,*]*1e-10,format=fmtstr
close,g & free_lun,g

;printing table of photometry by method II.
openw,g,app+'/Nov1_delivery/codes/photometry/tables/rp_phot.tab',/get_lun
fmtstr = '(A-29, x, F15.7, x, I3, x, A-11, 2(x, F6.2), x, I3, x, 2(x, F6.2), x, I1, x, F6.4, x, F11.2, x, F5.2, x, I3, 33(1x, E9.2E2))'
for i=0,ln-1 do printf,g,file[i],jd[i],doy[i],filter[i],xc_o[i],yc_o[i],frame[i],h_dist[i],v_dist[i],flag[i],hed[i],spc[i],pa[i],ap[i],phot_rp_pix[i,*]*1e-10,format=fmtstr
close,g & free_lun,g

;printing table of photometric error by method II.
openw,g,app+'/Nov1_delivery/codes/photometry/tables/rp_err.tab',/get_lun
fmtstr = '(A-29, x, F15.7, x, I3, x, A-11, 2(x, F6.2), x, I3, x, 2(x, F6.2), x, I1, x, F6.4, x, F11.2, x, F5.2, x, I3, 33(1x, E9.2E2))'
for i=0,ln-1 do printf,g,file[i],jd[i],doy[i],filter[i],xc_o[i],yc_o[i],frame[i],h_dist[i],v_dist[i],flag[i],hed[i],spc[i],pa[i],ap[i],e_rp_pix[i,*]*1e-10,format=fmtstr
close,g & free_lun,g

end
